@@ -11,6 +11,8 @@
 
 #include "pivot.h"
 
+enum { noWarning, warnZeroMAD };
+
 double median(double * x, int n, int copy, int * err);
 
 double quantile(double * x, int n, double q, int copy, int * err);
@@ -18,7 +20,7 @@ double quantile(double * x, int n, double q, int copy, int * err);
 void testMedian(double *x, int * n, double * res);
 
 void prepareColBicor(double * col, int nr, double maxPOutliers, int fallback, int cosine, double * res, 
-                     int * nNAentries, int * NAmed, int *zeroMAD, double * aux, double * aux2);
+                     int * nNAentries, int * NAmed, volatile int *zeroMAD, double * aux, double * aux2);
 
 void prepareColCor(double * x, int nr, int cosine, double * res, int * nNAentries, int * NAmean);
 
